import * as Yup from "yup";

export interface FormObject {
  name: string;
  label: string;
  placeholder?: string;
  type: "text" | "string" | "number" | "textarea" | "email" | "select" | "radioGroup" | "enumeration" | "checkbox" | "autocomplete" | "password" | "multi-select"| "date";
  values?: { value: string; label?: string }[];
  max?: number;
  match?: string;
  regex?: RegExp;
  matchMsg?: string;
  min?: number;
  defaultValue?: string | number | string[];
  required?: boolean;
}

const minLengthValidation = (value: string | undefined, limit: number | undefined = 3) => {
  if (value && value.length < limit) {
    return false;
  }
  return true;
};

const prepareValidationSchema = (formSchema: FormObject[]) => {
  let _formData: Record<string, any> = {};
  let _validationSchema: Record<string, any> = {};

  formSchema.forEach((object, index) => {
    if (object.name !== "submit") {
      _formData[object.name] = "";
    }

    if (object.type === "text" || object.type === "password" || object.type === "string" || object.type === "textarea" || object.type === "email") {
      if (object.hasOwnProperty("match")) {
        _validationSchema[object.name] = Yup.string()
          .label(object.label)
          .oneOf([Yup.ref(object.match), null], object.matchMsg);
      } else if (object.hasOwnProperty("regex")) {
        const regex = new RegExp(object.regex);
        _validationSchema[object.name] = Yup.string()
          .label(object.label)
          .matches(regex, object.matchMsg);
      } else {
        _validationSchema[object.name] = Yup.string().label(object.label);
      }

      if (object.hasOwnProperty("min")) {
        _validationSchema[object.name] = _validationSchema[object.name].min(object.min);
      }
      if (object.max) {
        _validationSchema[object.name] = _validationSchema[object.name].max(object.max);
      }
      _formData[object.name] = object.defaultValue ?? "";
    } else if (object.type === "select" || object.type === "radioGroup" || object.type === "enumeration") {
      const allOptions = object.values?.map((o) => o.value) || [];
      _validationSchema[object.name] = Yup.string().oneOf(allOptions).label(object.label);
      _formData[object.name] = object.defaultValue ?? allOptions[0];
    } else if (object.type === "multi-select" || object.type === "autocomplete") {
      _validationSchema[object.name] = Yup.array().label(object.label).of(Yup.string());
      if (object.required) {
        _validationSchema[object.name] = _validationSchema[object.name].min(1, `${object.label} is required`);
      }
      _formData[object.name] = object.defaultValue ?? [];
    } else if (object.type === "checkbox") {
      _validationSchema[object.name] = Yup.boolean().oneOf([true], `${object.label} is required`).label(object.label);
      _formData[object.name] = object.defaultValue ?? 0;
    } else if (object.type === "number") {
      _validationSchema[object.name] = Yup.number().label(object.label).typeError("Please enter a valid number");
      _formData[object.name] = object.defaultValue ?? "";
    }

    if (object.required) {
      if (object.type === "email") {
        _validationSchema[object.name] = (_validationSchema[object.name] as Yup.StringSchema<string>).email("Please provide a valid email").required("Email is a required field").label(object.label);
      } else if (object.type === "number" && (object.hasOwnProperty("max") || object.hasOwnProperty("min"))) {
        if (object.hasOwnProperty("min")) {
          _validationSchema[object.name] = _validationSchema[object.name].min(object.min, object.label + " must be greater than or equal to " + object.min);
        }
        if (object.max) {
          _validationSchema[object.name] = _validationSchema[object.name].max(object.max, object.label + " must be less than or equal to " + object.max);
        }
        _validationSchema[object.name] = _validationSchema[object.name].required();
      } else if (object.type === "multi-select" || object.type === "autocomplete") {
        _validationSchema[object.name] = _validationSchema[object.name].min(1).required();
      } else {
        _validationSchema[object.name] = _validationSchema[object.name].required();
      }
    }
  });

  return {
    validationSchema: Yup.object().shape(_validationSchema),
    initialFormData: _formData,
  };
};

export default prepareValidationSchema;
